@@ -14,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- * $Id: iLBC_decode.c,v 1.2 2008/03/06 12:27:38 steveu Exp $
+ * $Id: iLBC_decode.c,v 1.3 2008/10/02 12:00:56 steveu Exp $
  */
 
 /*! \file */
@@ -43,78 +43,88 @@
 /*----------------------------------------------------------------*
  *  Initiation of decoder instance.
  *---------------------------------------------------------------*/
-
-ilbc_decode_state_t *ilbc_decode_init(ilbc_decode_state_t *iLBCdec_inst,   /* (i/o) Decoder instance */
-                                      int mode,                            /* (i) frame size mode */
-                                      int use_enhancer)                    /* (i) 1 to use enhancer
-                                                                                  0 to run without enhancer */
+ilbc_decode_state_t *ilbc_decode_init(ilbc_decode_state_t *s,
+                                      int mode,
+                                      int use_enhancer)
 {
     int i;
 
-    iLBCdec_inst->mode = mode;
+    if (mode != 30  &&  mode != 20)
+        return NULL;
 
+    if (s == NULL)
+    {
+        if ((s = (ilbc_decode_state_t *) malloc(sizeof(*s))) == NULL)
+            return NULL;
+    }
+    memset(s, 0, sizeof(*s));
+
+    s->mode = mode;
     if (mode == 30)
     {
-        iLBCdec_inst->blockl = ILBC_BLOCK_LEN_30MS;
-        iLBCdec_inst->nsub = NSUB_30MS;
-        iLBCdec_inst->nasub = NASUB_30MS;
-        iLBCdec_inst->lpc_n = LPC_N_30MS;
-        iLBCdec_inst->no_of_bytes = ILBC_NO_OF_BYTES_30MS;
-        iLBCdec_inst->state_short_len = STATE_SHORT_LEN_30MS;
+        s->blockl = ILBC_BLOCK_LEN_30MS;
+        s->nsub = NSUB_30MS;
+        s->nasub = NASUB_30MS;
+        s->lpc_n = LPC_N_30MS;
+        s->no_of_bytes = ILBC_NO_OF_BYTES_30MS;
+        s->state_short_len = STATE_SHORT_LEN_30MS;
         /* ULP init */
-        iLBCdec_inst->ULP_inst = &ULP_30msTbl;
-    }
-    else if (mode == 20)
-    {
-        iLBCdec_inst->blockl = ILBC_BLOCK_LEN_20MS;
-        iLBCdec_inst->nsub = NSUB_20MS;
-        iLBCdec_inst->nasub = NASUB_20MS;
-        iLBCdec_inst->lpc_n = LPC_N_20MS;
-        iLBCdec_inst->no_of_bytes = ILBC_NO_OF_BYTES_20MS;
-        iLBCdec_inst->state_short_len = STATE_SHORT_LEN_20MS;
-        /* ULP init */
-        iLBCdec_inst->ULP_inst = &ULP_20msTbl;
+        s->ULP_inst = &ULP_30msTbl;
     }
     else
     {
-        return NULL;
+        s->blockl = ILBC_BLOCK_LEN_20MS;
+        s->nsub = NSUB_20MS;
+        s->nasub = NASUB_20MS;
+        s->lpc_n = LPC_N_20MS;
+        s->no_of_bytes = ILBC_NO_OF_BYTES_20MS;
+        s->state_short_len = STATE_SHORT_LEN_20MS;
+        /* ULP init */
+        s->ULP_inst = &ULP_20msTbl;
     }
 
-    memset(iLBCdec_inst->syntMem, 0, ILBC_LPC_FILTERORDER*sizeof(float));
-    memcpy((*iLBCdec_inst).lsfdeqold, lsfmeanTbl, ILBC_LPC_FILTERORDER*sizeof(float));
+    memset(s->syntMem, 0, ILBC_LPC_FILTERORDER*sizeof(float));
+    memcpy(s->lsfdeqold, lsfmeanTbl, ILBC_LPC_FILTERORDER*sizeof(float));
 
-    memset(iLBCdec_inst->old_syntdenum,
+    memset(s->old_syntdenum,
            0,
            ((ILBC_LPC_FILTERORDER + 1)*ILBC_NUM_SUB_MAX)*sizeof(float));
     for (i = 0;  i < ILBC_NUM_SUB_MAX;  i++)
-        iLBCdec_inst->old_syntdenum[i*(ILBC_LPC_FILTERORDER + 1)] = 1.0f;
+        s->old_syntdenum[i*(ILBC_LPC_FILTERORDER + 1)] = 1.0f;
 
-    iLBCdec_inst->last_lag = 20;
+    s->last_lag = 20;
 
-    iLBCdec_inst->prevLag = 120;
-    iLBCdec_inst->per = 0.0;
-    iLBCdec_inst->consPLICount = 0;
-    iLBCdec_inst->prevPLI = 0;
-    iLBCdec_inst->prevLpc[0] = 1.0f;
-    memset(iLBCdec_inst->prevLpc + 1, 0, ILBC_LPC_FILTERORDER*sizeof(float));
-    memset(iLBCdec_inst->prevResidual, 0, ILBC_BLOCK_LEN_MAX*sizeof(float));
-    iLBCdec_inst->seed = 777;
-    memset(iLBCdec_inst->hpomem, 0, 4*sizeof(float));
+    s->prevLag = 120;
+    s->per = 0.0;
+    s->consPLICount = 0;
+    s->prevPLI = 0;
+    s->prevLpc[0] = 1.0f;
+    memset(s->prevLpc + 1, 0, ILBC_LPC_FILTERORDER*sizeof(float));
+    memset(s->prevResidual, 0, ILBC_BLOCK_LEN_MAX*sizeof(float));
+    s->seed = 777;
+    memset(s->hpomem, 0, 4*sizeof(float));
 
-    iLBCdec_inst->use_enhancer = use_enhancer;
-    memset(iLBCdec_inst->enh_buf, 0, ENH_BUFL*sizeof(float));
+    s->use_enhancer = use_enhancer;
+    memset(s->enh_buf, 0, ENH_BUFL*sizeof(float));
     for (i = 0;  i < ENH_NBLOCKS_TOT;  i++)
-        iLBCdec_inst->enh_period[i] = 40.0f;
+        s->enh_period[i] = 40.0f;
 
-    iLBCdec_inst->prev_enh_pl = 0;
+    s->prev_enh_pl = 0;
 
-    return iLBCdec_inst;
+    return s;
 }
+/*- End of function --------------------------------------------------------*/
+
+int ilbc_decode_release(ilbc_decode_state_t *s)
+{
+    free(s);
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
 
 /*----------------------------------------------------------------*
  *  frame residual decoder function (subrutine to iLBC_decode)
  *---------------------------------------------------------------*/
-
 static void Decode(ilbc_decode_state_t *iLBCdec_inst,  /* (i/o) the decoder state structure */
                    float *decresidual,                 /* (o) decoded residual frame */
                    int start,                          /* (i) location of start state */
@@ -278,11 +288,11 @@ static void Decode(ilbc_decode_state_t *iLBCdec_inst,  /* (i/o) the decoder stat
             decresidual[SUBL*Nback - i - 1] = reverseDecresidual[i];
     }
 }
+/*- End of function --------------------------------------------------------*/
 
 /*----------------------------------------------------------------*
  *  main decoder function
  *---------------------------------------------------------------*/
-
 static void ilbc_decode_frame(ilbc_decode_state_t *iLBCdec_inst, /* (i/o) the decoder state structure */
                               float decblock[],                  /* (o) decoded signal block */
                               const uint8_t bytes[],             /* (i) encoded signal bits */
@@ -566,6 +576,7 @@ static void ilbc_decode_frame(ilbc_decode_state_t *iLBCdec_inst, /* (i/o) the de
         iLBCdec_inst->prev_enh_pl = 1;
     }
 }
+/*- End of function --------------------------------------------------------*/
 
 int ilbc_decode(ilbc_decode_state_t *s,     /* (i/o) the decoder state structure */
                 int16_t amp[],              /* (o) decoded signal block */
@@ -595,6 +606,7 @@ int ilbc_decode(ilbc_decode_state_t *s,     /* (i/o) the decoder state structure
     }
     return i;
 }
+/*- End of function --------------------------------------------------------*/
 
 int ilbc_fillin(ilbc_decode_state_t *s,     /* (i/o) the decoder state structure */
                 int16_t amp[],              /* (o) decoded signal block */
@@ -623,3 +635,5 @@ int ilbc_fillin(ilbc_decode_state_t *s,     /* (i/o) the decoder state structure
     }
     return i;
 }
+/*- End of function --------------------------------------------------------*/
+/*- End of file ------------------------------------------------------------*/
