@@ -14,7 +14,7 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  *
- * $Id: iLBC_encode.c,v 1.2 2008/03/06 12:27:38 steveu Exp $
+ * $Id: iLBC_encode.c,v 1.3 2008/10/02 12:00:56 steveu Exp $
  */
 
 /*! \file */
@@ -45,51 +45,62 @@
 /*----------------------------------------------------------------*
  *  Initiation of encoder instance.
  *---------------------------------------------------------------*/
-
-ilbc_encode_state_t *ilbc_encode_init(ilbc_encode_state_t *iLBCenc_inst, /* (i/o) Encoder instance */
-                                      int mode)                          /* (i) frame size mode */
+ilbc_encode_state_t *ilbc_encode_init(ilbc_encode_state_t *s, int mode)
 {
-    iLBCenc_inst->mode = mode;
+    if (mode != 30  &&  mode != 20)
+        return NULL;
+
+    if (s == NULL)
+    {
+        if ((s = (ilbc_encode_state_t *) malloc(sizeof(*s))) == NULL)
+            return NULL;
+    }
+    memset(s, 0, sizeof(*s));
+
+    s->mode = mode;
     if (mode == 30)
     {
-        iLBCenc_inst->blockl = ILBC_BLOCK_LEN_30MS;
-        iLBCenc_inst->nsub = NSUB_30MS;
-        iLBCenc_inst->nasub = NASUB_30MS;
-        iLBCenc_inst->lpc_n = LPC_N_30MS;
-        iLBCenc_inst->no_of_bytes = ILBC_NO_OF_BYTES_30MS;
-        iLBCenc_inst->state_short_len = STATE_SHORT_LEN_30MS;
+        s->blockl = ILBC_BLOCK_LEN_30MS;
+        s->nsub = NSUB_30MS;
+        s->nasub = NASUB_30MS;
+        s->lpc_n = LPC_N_30MS;
+        s->no_of_bytes = ILBC_NO_OF_BYTES_30MS;
+        s->state_short_len = STATE_SHORT_LEN_30MS;
         /* ULP init */
-        iLBCenc_inst->ULP_inst = &ULP_30msTbl;
-    }
-    else if (mode == 20)
-    {
-        iLBCenc_inst->blockl = ILBC_BLOCK_LEN_20MS;
-        iLBCenc_inst->nsub = NSUB_20MS;
-        iLBCenc_inst->nasub = NASUB_20MS;
-        iLBCenc_inst->lpc_n = LPC_N_20MS;
-        iLBCenc_inst->no_of_bytes = ILBC_NO_OF_BYTES_20MS;
-        iLBCenc_inst->state_short_len = STATE_SHORT_LEN_20MS;
-        /* ULP init */
-        iLBCenc_inst->ULP_inst = &ULP_20msTbl;
+        s->ULP_inst = &ULP_30msTbl;
     }
     else
     {
-        return NULL;
+        s->blockl = ILBC_BLOCK_LEN_20MS;
+        s->nsub = NSUB_20MS;
+        s->nasub = NASUB_20MS;
+        s->lpc_n = LPC_N_20MS;
+        s->no_of_bytes = ILBC_NO_OF_BYTES_20MS;
+        s->state_short_len = STATE_SHORT_LEN_20MS;
+        /* ULP init */
+        s->ULP_inst = &ULP_20msTbl;
     }
 
-    memset((*iLBCenc_inst).anaMem, 0, ILBC_LPC_FILTERORDER*sizeof(float));
-    memcpy((*iLBCenc_inst).lsfold, lsfmeanTbl, ILBC_LPC_FILTERORDER*sizeof(float));
-    memcpy((*iLBCenc_inst).lsfdeqold, lsfmeanTbl, ILBC_LPC_FILTERORDER*sizeof(float));
-    memset((*iLBCenc_inst).lpc_buffer, 0, (LPC_LOOKBACK + ILBC_BLOCK_LEN_MAX)*sizeof(float));
-    memset((*iLBCenc_inst).hpimem, 0, 4*sizeof(float));
+    memset(s->anaMem, 0, ILBC_LPC_FILTERORDER*sizeof(float));
+    memcpy(s->lsfold, lsfmeanTbl, ILBC_LPC_FILTERORDER*sizeof(float));
+    memcpy(s->lsfdeqold, lsfmeanTbl, ILBC_LPC_FILTERORDER*sizeof(float));
+    memset(s->lpc_buffer, 0, (LPC_LOOKBACK + ILBC_BLOCK_LEN_MAX)*sizeof(float));
+    memset(s->hpimem, 0, 4*sizeof(float));
 
-    return iLBCenc_inst;
+    return s;
 }
+/*- End of function --------------------------------------------------------*/
+
+int ilbc_encode_release(ilbc_encode_state_t *s)
+{
+    free(s);
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
 
 /*----------------------------------------------------------------*
  *  main encoder function
  *---------------------------------------------------------------*/
-
 static int ilbc_encode_frame(ilbc_encode_state_t *iLBCenc_inst,     /* (i/o) the general encoder state */
                              uint8_t bytes[],                       /* (o) encoded data bits iLBC */
                              const float block[])                   /* (o) speech vector to encode */
@@ -495,6 +506,7 @@ static int ilbc_encode_frame(ilbc_encode_state_t *iLBCenc_inst,     /* (i/o) the
     dopack(&pbytes, 0, 1, &pos);
     return iLBCenc_inst->no_of_bytes;
 }
+/*- End of function --------------------------------------------------------*/
 
 int ilbc_encode(ilbc_encode_state_t *s,     /* (i/o) the general encoder state */
                 uint8_t bytes[],            /* (o) encoded data bits iLBC */
@@ -515,3 +527,5 @@ int ilbc_encode(ilbc_encode_state_t *s,     /* (i/o) the general encoder state *
     }
     return j;
 }
+/*- End of function --------------------------------------------------------*/
+/*- End of file ------------------------------------------------------------*/
